import tempfile
import os

import pandas as pd
from airflow.providers.postgres.hooks.postgres import PostgresHook


class PostgresLoader:

    def __init__(self, pg_conn_id):
        self.__pg_conn_id = pg_conn_id

    def __clearTable(self, tableName, schema):
        pgHook = PostgresHook(postgres_conn_id=self.__pg_conn_id)
        sql = f'delete from {schema}.{tableName}'
        pgHook.run(sql=sql)

    def copy_data(self, tuples, tableName, schema, columns, clear=False):
        df = pd.DataFrame(data=tuples, columns=columns);

        pgHook = PostgresHook(postgres_conn_id=self.__pg_conn_id)
        if clear == True:
            self.__clearTable(tableName=tableName, schema=schema)

        tmpFile = tempfile.NamedTemporaryFile(suffix='.csv');
        tmpPath = os.path.realpath(tmpFile.name)

        #each column should be incased in "" to avoid naming conflicts name->"name"
        columnsJoined = ','.join('"'+columnName.lower()+'"' for columnName in columns)


        print(columnsJoined)
        print(df)
        df.to_csv(tmpFile, index=False, header=True)


        sql = f"COPY {schema}.{tableName} ({columnsJoined}) FROM STDIN DELIMITER ',' CSV HEADER"
        print(sql)
        conn = pgHook.get_conn();
        cursor = conn.cursor()
        result = cursor.copy_expert(sql, open(tmpPath, "r"))
        print("StatusMessage: ",cursor.statusmessage)
        print("result: ", result)
        conn.commit()
        conn.close()

    def import_data(self, rows, colNames, tableName, schema, clear=False):
        pgHook = PostgresHook(postgres_conn_id=self.__pg_conn_id)
        if clear == True:
            self.__clearTable(tableName=tableName, schema=schema)
        # sql = PostgresHook._generate_insert_sql(table=schema + '.' +tableName, values=rows[0], target_fields=colNames, replace=False)

        #each column should be incased in "" to avoid naming conflicts name->"name"
        columnsJoined = ','.join('"'+columnName.lower()+'"' for columnName in colNames)
        placeHolderJoined = ', '.join(list(map(lambda x: '%s', range(len(colNames)))))
        sql = f'insert into {schema}.{tableName}({columnsJoined}) values({placeHolderJoined})'
        for row in rows:
            pgHook.run(sql=sql, parameters=row)
        return 0

    def run_function(self, functionName, schema, parameters=[]):
        pgHook = PostgresHook(postgres_conn_id=self.__pg_conn_id)
        placeHolderJoined = ', '.join(list(map(lambda x: '%s', range(len(parameters)))))
        sql = f'select {schema}.{functionName}({placeHolderJoined})'
        result = pgHook.get_first(sql=sql, parameters=parameters)
        return result

    def load_to_tuples(self, tableName, colNames, schema, where=None):
        """
        Reads data from a Postgres table and returns it as a list of tuples.

        :param tableName: The name of the table to read from.
        :param colNames: A list of strings representing the column names to read.
        :param schema: The schema of the table.
        :param where: OPTIONAL. A list of objects representing the column, symbol, and value to filter by.
                    Example: [{ 'column': 'bool', 'symbol': '=', 'value': 0 }, { 'column': 'desc', 'symbol': '<>', 'value': 'test' },{
                                                                                                                                             "column": "desc",
                                                                                                                                             "symbol": "<>",
                                                                                                                                             "value": "\"4\""
                                                                                                                                           }]
                    ATTENTION: types need to match!

        :return: A list of tuples representing the rows in the table.
        """

        # Initialize a PostgresHook to connect to the database
        pgDbHook = PostgresHook(self.__pg_conn_id)

        # Create a string with the lowercase column names separated by commas
        columns = ",".join('"' + columnName.lower() + '"' for columnName in colNames)

        # If a where clause is specified, construct the SQL WHERE clause with the filter conditions + check if value is string or number....
        if len(where) > 0:
            whereClause = "WHERE " + " AND ".join('"' + cond['column'].lower() + '"' + ' ' + cond['symbol'] + ' ' +
                                                (f"'{cond['value']}'" if not str(cond['value']).isdigit() else str(cond['value']))
                                                for cond in where)
        else:
            whereClause = ""

        # Construct the SQL query with the specified table, columns, and WHERE clause
        sql = f"SELECT {columns} FROM {schema}.{tableName} {whereClause}"

        # Execute the SQL query and retrieve the data as a Pandas DataFrame
        df = pgDbHook.get_pandas_df(sql)
        print("Used SQL statement: ", sql)

        # Convert the DataFrame to a list of tuples representing the rows in the table
        return list(df.itertuples(index=False))



    def get_column_names(self, tableName, schema):
        pgHook = PostgresHook(postgres_conn_id=self.__pg_conn_id)
        sql = f'''
        SELECT column_name
            FROM information_schema.columns
            WHERE table_schema = '{schema}'
            AND table_name   = '{tableName}'
            order by ordinal_position;
        '''
        df = pgHook.get_pandas_df(sql=sql)
        columnNames = df['column_name'].tolist()
        print(columnNames)
        return columnNames




from airflow.providers.jdbc.hooks.jdbc import JdbcHook
import datetime

#from java.sql import Timestamp

class Compass:

    def __init__(self, compass_conn_id):
        self.__compass_conn_id = compass_conn_id

    def __dataFrameToTuples(self, df, columns):
        tuples= []
        for i, row in df.iterrows():
            tuple = ()

            for idx, column in enumerate(columns):
                try:
                    timestamp = row[idx].getTime() / 1000
                    tuple = tuple + (datetime.datetime.fromtimestamp(timestamp).astimezone().isoformat(),)
                except AttributeError:
                    tuple = tuple + (row[idx],)
            tuples.append(tuple)

        return tuples


    def load_to_dataframe(self, tableName, columns, schema, where=[]):
        cpHook = JdbcHook(log_sql=True, jdbc_conn_id=self.__compass_conn_id)
        strColumns = ",".join('"' + columnName.upper() + '"' for columnName in columns)

        if len(where) > 0:
            whereClause = "WHERE " + " AND ".join('"' + cond['column'].upper() + '"' + ' ' + cond['symbol'] + ' ' +
                                                  (f"'{cond['value']}'" if not str(cond['value']).isdigit() else str(
                                                      cond['value']))
                                                  for cond in where)
        else:
            whereClause = ""

        sql = f'select {strColumns} from {schema}.{tableName} {whereClause}'
        # print(sql)
        db = cpHook.get_pandas_df(sql=sql)
        # print(db)
        return db

    def load_to_tuples(self, tableName, columns, schema, where=[]):
        df = self.load_to_dataframe(tableName=tableName, columns=columns, schema=schema)
        tuples = self.__dataFrameToTuples(df=df, columns=columns)
        # print(tuples)
        return tuples
        # return list(df.itertuples(index=False, name=None))


import json

from airflow.providers.http.hooks.http import HttpHook
from airflow.exceptions import AirflowException
from airflow.exceptions import AirflowBadRequest

from json import JSONDecodeError


class Planio:
    __customFields = [
        {'column': 'project_status', 'name': 'Project Status'},
        {'column': 'customer', 'name': 'Customer'},
        {'column': 'to_invoice', 'name': 'to invoice'},
        {'column': 'invoiced', 'name': 'invoiced'},
        {'column': 'additional_requirement', 'name': 'Additional requirement'},
    ]

    __refrenceFields = [
        {'column': 'project_id', 'ref': ['project', 'id']},
        {'column': 'user_id', 'ref': ['user', 'id']},
        {'column': 'activity_id', 'ref': ['activity', 'id']},
        {'column': 'activity_name', 'ref': ['activity', 'name']},
        {'column': 'tracker_id', 'ref': ['tracker', 'id']},
        {'column': 'tracker_name', 'ref': ['tracker', 'name']},
        {'column': 'status_id', 'ref': ['status', 'id']},
        {'column': 'status_name', 'ref': ['status', 'name']},
        {'column': 'priority_id', 'ref': ['priority', 'id']},
        {'column': 'issue_id', 'ref': ['issue', 'id']},
        {'column': 'priority_name', 'ref': ['priority', 'name']},
        {'column': 'author_id', 'ref': ['author', 'id']},
        {'column': 'assigned_to', 'ref': ['assigned_to', 'id']},
        {'column': 'fixed_version_id', 'ref': ['fixed_version', 'id']},
    ]

    userColumns = ['id', 'login', 'firstname', 'lastname', 'mail']
    projectColumns = ['id', 'name', 'identifier', 'description', 'customer', 'status']
    timeEntryColumns = ['id', 'issue_id', 'user_id', 'activity_id', 'activity_name', 'hours', 'comments', 'spent_on']
    versionColumns = ['id', 'project_id', 'name', 'description', 'status', 'due_date', 'to_invoice', 'invoiced']
    issueColumns = ['id', 'project_id', 'tracker_id', 'tracker_name', 'fixed_version_id', 'status_id', 'status_name', 'priority_id', 'priority_name', 'author_id', 'assigned_to', 'subject', 'description', 'estimated_hours', 'spent_hours', 'additional_requirement']
    customFieldColumns = ['value', 'label']

    def __init__(self, planio_connection_id):
        self.__planio_connection_id = planio_connection_id

    def __toTuple(self, item, columns):
        tuple = ();
        for column in columns:
            customField = list(filter(lambda cf: cf['column'] == column, self.__customFields))
            referenceField = list(filter(lambda cf: cf['column'] == column, self.__refrenceFields))
            if len(customField) > 0:
                try:
                    customValue = list(filter(lambda x: x["name"] == customField[0]["name"], item["custom_fields"]));
                    if (len(customValue) > 0):
                        tuple += (customValue[0]["value"],);
                    else:
                        tuple += (None,)
                except (KeyError):
                    tuple += (None,)

            elif (len(referenceField) > 0):
                value = item
                for prop in referenceField[0]['ref']:
                    try:
                        value = value[prop]
                    except (KeyError):
                        value = None
                        break
                tuple += (value,)
            else:
                tuple += (item[column],)
        return tuple

    def __toTuples(self, arr, columns):
        return list(map(lambda item: self.__toTuple(item=item, columns=columns), arr))

    def __read_pages(self, endpoint, resultProperty, offset, limit):
        pagedEndpoint = f'{endpoint}&limit={limit}&offset={offset}'
        http = HttpHook(method="GET", http_conn_id=self.__planio_connection_id)
        response = http.run(pagedEndpoint)
        try:
            jsonO = json.loads(response.content)
            data = jsonO[resultProperty];
            totalCount = jsonO['total_count']
            if totalCount > offset + limit:
                offset = offset + limit
                data += self.__read_pages(endpoint=endpoint, resultProperty=resultProperty, offset=offset, limit=limit)
            return data
        except (JSONDecodeError, LookupError, AirflowException) as ex:
            print(ex)
            raise AirflowBadRequest(ex)

    def __loadProperty(self, property, columns, query=None):
        endpoint = f'/{property}.json?encoding=UTF-8'
        if query != None:
            endpoint += query
        limit = 100
        offset = 0;
        data = self.__read_pages(endpoint=endpoint, resultProperty=property, offset=offset, limit=limit)
        return self.__toTuples(arr=data, columns=columns)


    def loadUsers(self, columns):
        return self.__loadProperty(property='users', columns=columns)

    def loadProjects(self, columns):
        return self.__loadProperty(property='projects', columns=columns)

    def loadIssues(self, columns):
        query = "&status_id=*&created_on=%3E%3D2022-01-01"
        return self.__loadProperty(property='issues', columns=columns, query=query)

    def loadTimeEntries(self, columns, date):
        query = "&spent_on={}".format(date)
        return self.__loadProperty(property='time_entries', columns=columns, query=query)

    def __loadVersionsOfProject(self, columns, projectId):
        endpoint = f'/projects/{projectId}/versions.json?encoding=UTF-8'
        resultProperty = 'versions'
        limit = 100
        offset = 0;
        data = self.__read_pages(endpoint=endpoint, resultProperty=resultProperty, offset=offset, limit=limit)
        return self.__toTuples(arr=data, columns=columns)

    def loadVersions(self, columns):
        versions = [];
        for tuple in self.loadProjects(columns=['id', 'status']):
            if (int(tuple[1] == 1)):
                try:
                    versions += self.__loadVersionsOfProject(columns=columns, projectId=tuple[0])
                except (AirflowException):
                    pass
        return versions

    def loadCustomFields(self, customFieldName):
        endpoint = '/custom_fields.json?encoding=UTF-8'
        http = HttpHook(method="GET", http_conn_id=self.__planio_connection_id)
        response = http.run(endpoint)
        try:
            jsonO = json.loads(response.content)['custom_fields']
            print('-----------')
            print(jsonO)
            iterable = list(jsonO)
            data = list(filter(lambda cf: cf['name'] == 'Customer', iterable))
            #customField = list(filter(lambda cf: cf['column'] == column, self.__customFields))

            self.__toTuples(arr=data[0]['possible_values'], columns=self.customFieldColumns)
        except (JSONDecodeError, LookupError, AirflowException) as ex:
            print(ex)
            raise AirflowBadRequest(ex)



import json

from airflow.providers.http.hooks.http import HttpHook
from airflow.exceptions import AirflowException
from airflow.exceptions import AirflowBadRequest

from json import JSONDecodeError


class Hubspot:

    def __init__(self, hubspot_conn_id):
        self.__hubspot_conn_id = hubspot_conn_id

    def __toTuple(self, item, colNames):
        tuple = (item['id'],)

        for colName in colNames:
            try:
                tuple = tuple + (item['properties'][colName],)
            except KeyError:
                tuple = tuple + ('UNDEFINED',)

        return tuple

    def __labelToTuple(self, labelType, item):
        tuple = (labelType,)
        if item['value'] == "":
            tuple = tuple + ('UNDEFINED',)
        else:
            tuple = tuple + (item['value'],)
        if item['label'] == '':
            tuple = tuple + ('UNDEFINED',)
        else:
            tuple = tuple + (item['label'],)
        return tuple

    def __read_page(self, endpoint, resultProperty, after=None):
        pagedEndpoint = endpoint
        if after != None:
            pagedEndpoint += '&after=' + after
        response = HttpHook(method="GET", http_conn_id=self.__hubspot_conn_id).run(pagedEndpoint)
        try:
            jsonO = json.loads(response.content)
            data = jsonO[resultProperty];
            if 'paging' in jsonO:
                after = jsonO['paging']['next']['after']
                data += self.__read_page(endpoint=endpoint, resultProperty=resultProperty, after=after)
            return data
        except (JSONDecodeError, LookupError, AirflowException) as ex:
            print(ex)
            raise AirflowBadRequest(ex)

    def __companyToTuple(self, item, colNames):
        tuple = (item['id'],)

        for colName in colNames:
            try:
                if item['properties'][colName] == '':
                    tuple = tuple + (None,)
                else:
                    tuple = tuple + (item['properties'][colName],)
            except KeyError:
                tuple = tuple + ('UNDEFINED',)

        try:
            companies = item["associations"]["companies"]["results"]
            companyId = [*filter(lambda x: x["type"] == "deal_to_company", companies)][0]["id"]
        except (KeyError, IndexError):
            companyId = None
        tuple += (companyId,)
        return tuple

    def __stageToTuple(self, item):
        tuple = (item['stageId'], item['label'], item['metadata']['isClosed'], item['metadata']['probability'])
        return tuple

    def __ownerToTuple(self, item):
        tuple = (item['id'], item['email'], item['firstName'], item['lastName'], item['userId'])
        try:
            tuple = tuple + (item['teams'][0]['id'],)
        except KeyError:
            tuple = tuple + ('UNDEFINED',)

        try:
            tuple = tuple + (item['teams'][0]['name'],)
        except KeyError:
            tuple = tuple + ('UNDEFINED',)
        return tuple

    def read_deals(self, colNames):


        dealsEndpoint = "/crm/v4/objects/deals?limit=100&properties=critical_opp_review&properties=dealname&properties=dealType&properties=dealstage&properties=amount&properties=deal_currency_code&properties=hy2_market&properties=hy2_product&properties=hy2_application&properties=hubspot_owner_id&properties=hs_deal_stage_probability&properties=hs_exchange_rate&properties=of_systems&properties=expected_delivery&properties=h2_kg_&properties=hy2_battery&properties=hy2_storage&properties=hs_object_id&properties=electrolyzer&properties=sales_team&properties=hy_invoice_co_&properties=hy_mfg_co_&associations=companies"
        dealList = self.__read_page(endpoint=dealsEndpoint, resultProperty='results')
        try:
            data = [*map(lambda x: self.__companyToTuple(x, colNames), dealList)]
            return data

        except (JSONDecodeError, LookupError, AirflowException) as ex:
            print(ex)
            raise AirflowBadRequest(ex)

    def read_companies(self, colNames):
        companiesEndpoint = "/crm/v3/objects/companies?limit=100&properties=city&properties=name&properties=country&properties=industry&properties=state&properties=type"
        companiesList = self.__read_page(endpoint=companiesEndpoint, resultProperty='results')
        try:
            data = [*map(lambda x: self.__toTuple(x, colNames), companiesList)]
            return data

        except (JSONDecodeError, LookupError, AirflowException) as ex:
            print(ex)
            raise AirflowBadRequest(ex)

    def read_stages(self):
        pipelineEndpoint = "/crm-pipelines/v1/pipelines/deals"
        response = HttpHook(method="GET", http_conn_id=self.__hubspot_conn_id).run(pipelineEndpoint)
        try:
            stagesList = json.loads(response.content)["results"][0]["stages"]
            data = [*map(lambda x: self.__stageToTuple(x), stagesList)]
            return data

        except (JSONDecodeError, LookupError, AirflowException) as ex:
            print(ex)
            raise AirflowBadRequest(ex)

    def read_owners(self):
        pipelineEndpoint = "/crm/v3/owners?limit=100"
        response = HttpHook(method="GET", http_conn_id=self.__hubspot_conn_id).run(pipelineEndpoint)
        try:
            ownersList = json.loads(response.content)["results"]
            data = [*map(lambda x: self.__ownerToTuple(x), ownersList)]
            return data

        except (JSONDecodeError, LookupError, AirflowException) as ex:
            print(ex)
            raise AirflowBadRequest(ex)

    def read_labels(self):
        dealsPropertiesEndpoint = "/crm/v3/properties/deals"
        dealsResponse = HttpHook(method="GET", http_conn_id=self.__hubspot_conn_id).run(dealsPropertiesEndpoint)
        companiesPropertiesEndpoint = "/crm/v3/properties/companies"
        companiesResponse = HttpHook(method="GET", http_conn_id=self.__hubspot_conn_id).run(companiesPropertiesEndpoint)
        try:
            dealsResult = json.loads(dealsResponse.content)["results"]
            hy2Applications = [*filter(lambda x: x['name'] == "hy2_application", dealsResult)][0]['options']
            hy2Markets = [*filter(lambda x: x['name'] == "hy2_market", dealsResult)][0]['options']
            dealtypes = [*filter(lambda x: x['name'] == "dealtype", dealsResult)][0]['options']
            hy2Products = [*filter(lambda x: x['name'] == "hy2_product", dealsResult)][0]['options']
            expectedDeliveries = [*filter(lambda x: x['name'] == "expected_delivery", dealsResult)][0]['options']
            hy2Batteries = [*filter(lambda x: x['name'] == "hy2_battery", dealsResult)][0]['options']
            hy2Storages = [*filter(lambda x: x['name'] == "hy2_storage", dealsResult)][0]['options']
            electrolyzers = [*filter(lambda x: x['name'] == "electrolyzer", dealsResult)][0]['options']
            salesteams = [*filter(lambda x: x['name'] == "sales_team", dealsResult)][0]['options']

            companiesResult = json.loads(companiesResponse.content)["results"]
            industries = [*filter(lambda x: x['name'] == "industry", companiesResult)][0]['options']
            types = [*filter(lambda x: x['name'] == "type", companiesResult)][0]['options']

            data = [*map(lambda x: self.__labelToTuple('hy2_application', x), hy2Applications)]
            data = data + [*map(lambda x: self.__labelToTuple('hy2_market', x), hy2Markets)]
            data = data + [*map(lambda x: self.__labelToTuple('dealtype', x), dealtypes)]
            data = data + [*map(lambda x: self.__labelToTuple('hy2_product', x), hy2Products)]
            data = data + [*map(lambda x: self.__labelToTuple('expected_delivery', x), expectedDeliveries)]
            data = data + [*map(lambda x: self.__labelToTuple('hy2_battery', x), hy2Batteries)]
            data = data + [*map(lambda x: self.__labelToTuple('hy2_storage', x), hy2Storages)]
            data = data + [*map(lambda x: self.__labelToTuple('electrolyzer', x), electrolyzers)]
            data = data + [*map(lambda x: self.__labelToTuple('salesteam', x), salesteams)]

            data = data + [*map(lambda x: self.__labelToTuple('industry', x), industries)]
            data = data + [*map(lambda x: self.__labelToTuple('type', x), types)]
            return data

        except (JSONDecodeError, LookupError, AirflowException) as ex:
            print(ex)
            raise AirflowBadRequest(ex)




from airflow.providers.oracle.hooks.oracle import OracleHook

class OracleLoader:
    def __init__(self, ora_conn_id):
        """
        Constructor that initializes an instance of OracleLoader with a connection ID to an Oracle database.

        :param ora_conn_id: The ID of the Airflow connection to the Oracle database.
        """
        self.ora_conn_id = ora_conn_id
        # Create an OracleHook instance to connect to the database.
        self.ora_hook = OracleHook(oracle_conn_id=self.ora_conn_id)

    def read_interface_from_oracle(self, interface_def):
        """
        Reads data from an Oracle database using a SQL statement specified in the interface definition and returns it as a JSON object.

        :param interface_def: A dictionary containing the interface definition, with the following keys:
            - connectionId: The ID of the Airflow connection to the Oracle database.
            - stmt: The SQL statement to execute.
            - columns: A dictionary mapping the column names in the result set to new column names.

        :return: A JSON object containing the result set.
        """
        # Execute the SQL statement and get a pandas DataFrame with the result set.
        df = self.ora_hook.get_pandas_df(sql=interface_def['stmt'])
        # Rename the columns based on the mapping specified in the interface definition.
        df = df.rename(columns=interface_def['columns'])
        # Convert the DataFrame to a JSON object and return it.
        return df.to_json(orient='records')

    def clear_table(self, tableName):
        """
        Clears all data from a table in the Oracle database.

        :param table_name: The name of the table to clear.
        """
        # Construct the SQL statement to delete all rows from the table.
        sql = f'DELETE FROM {tableName}'
        # Execute the SQL statement.
        self.ora_hook.run(sql=sql)

    def copy_data(self, tuples, tableName, columns, clear=False, bulk=True):
        """
        Inserts a bulk of rows into an Oracle table.

        :param tuples: A list of tuples representing the rows to insert.
        :param table_name: The name of the table to insert the rows into.
        :param columns: A list of strings representing the column names in the table.
        :param clear: A boolean indicating whether to clear the table before inserting the rows.

        :return: 0 if successful.
        """
        # If clear is True, then clear the table before inserting the rows.
        if clear:
            self.clear_table(tableName=tableName)
        # If there are rows to insert, then bulk-insert them using the OracleHook.
        if tuples:
            if bulk:
                self.ora_hook.bulk_insert_rows(table=tableName, rows=tuples, target_fields=columns)
            else:
                self.ora_hook.insert_rows(table=tableName, rows=tuples, target_fields=columns)
        # Return 0 to indicate success.
        return 0

    def load_to_tuples(self, tableName, colNames,where=[]):
        """
        Reads data from an Oracle table and returns it as a list of tuples.

        :param tableName: The name of the table to read from.
        :param colNames: A list of strings representing the column names to read.
        :param where: OPTIONAL. A list of objects representing the column, symbol, and value to filter by.
                    Example: [{ 'column': 'bool', 'symbol': '=', 'value': 0 }, { 'column': 'desc', 'symbol': '<>', 'value': 'test' },{
                                                                                                                                             "column": "desc",
                                                                                                                                             "symbol": "<>",
                                                                                                                                             "value": "\"4\""
                                                                                                                                           }]
                    ATTENTION: types need to match!

        :return: A list of tuples representing the rows in the table.
        """
        if(len(colNames)==0):
            print('No Columns given, return 0')
            return 0
        # Create a string with the uppercase column names separated by commas
        columns = ",".join('"' + columnName.upper() + '"' for columnName in colNames)

        # If a where clause is specified, construct the SQL WHERE clause with the filter conditions + check if value is string or number....
        if len(where) > 0:
            whereClause = "WHERE " + " AND ".join('"' + cond['column'].upper() + '"' + ' ' + cond['symbol'] + ' ' +
                                                (f"'{cond['value']}'" if not str(cond['value']).isdigit() else str(cond['value']))
                                                for cond in where)
        else:
            whereClause = ""

        # Construct the SQL query with the specified table, columns, and WHERE clause
        sql = f"SELECT {columns} FROM {tableName} {whereClause}"

        # Execute the SQL query and retrieve the data as a Pandas DataFrame
        df = self.ora_hook.get_pandas_df(sql)
        print("Used SQL statement: ", sql)

        # Convert the DataFrame to a list of tuples representing the rows in the table
        return list(df.itertuples(index=False))


    def get_column_names(self, tableName):
        """
        Gets all column names of the given table using COLUMN_NAME

        :param table_name: The name of the table to read from.

        :return: A list of ColumnNames representing the columns in the table.
        """
        print(f"Finding Oracle columns with: {tableName}")
        columns_query = f"SELECT COLUMN_NAME FROM ALL_TAB_COLUMNS WHERE TABLE_NAME = '{tableName.upper()}'"
        with self.ora_hook.get_conn() as conn:
            with conn.cursor() as cur:
                cur.execute(columns_query)
                column_names = [row[0] for row in cur.fetchall()]
        return column_names





from setuptools import setup

setup(
    name='abuscom-libs',
    version='0.1.12',
    description='Utility classes for airflow DAGs',
    url='https://abuscom.com',
    author='Leonhard Holzer',
    author_email='leonhard.holzer@abuscom.com',
    license='BSD 2-clause',
    packages=['abuscom.connectors'],
    install_requires=['apache-airflow>=2.4.3',
                      'pandas',
                      ],

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: Freeware',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 3.8',
    ],
)